@@ -1,25 +1,12 @@
 #============================================================================================================
 #
-#	ANZX[UǗW[(FARAMIR)
-#	faramir.pl
-#	------------------------------------------
-#	2002.12.15 start
-#	2003.01.22 ʃC^tFCXɈڍs
-#	2003.02.25 ύX
-#------------------------------------------------------------------------------------------------------------
-#
-#	Load
-#	Save
-#	Set
-#	Get
-#	Clear
-#	Check
+#	ANZX[UǗW[
 #
 #============================================================================================================
 package	FARAMIR;
 
 use strict;
-use warnings;
+#use warnings;
 
 #------------------------------------------------------------------------------------------------------------
 #
@@ -31,15 +18,15 @@ use warnings;
 #------------------------------------------------------------------------------------------------------------
 sub new
 {
-	my $this = shift;
-	my (@USER, $obj);
+	my $class = shift;
 	
-	$obj = {
-		'TYPE'		=> '',
-		'METHOD'	=> '',
-		'USER'		=> \@USER
+	my $obj = {
+		'TYPE'		=> undef,
+		'METHOD'	=> undef,
+		'USER'		=> undef,
+		'SYS'		=> undef,
 	};
-	bless $obj, $this;
+	bless $obj, $class;
 	
 	return $obj;
 }
@@ -48,7 +35,7 @@ sub new
 #
 #	[Uf[^ǂݍ - Load
 #	-------------------------------------------
-#	@F$M : MELKOR
+#	@F$Sys : MELKOR
 #	߂lFǂݍ:0,G[:1
 #
 #------------------------------------------------------------------------------------------------------------
@@ -56,28 +43,23 @@ sub Load
 {
 	my $this = shift;
 	my ($Sys) = @_;
-	my (@datas, @head, $path, $dummy);
 	
 	$this->{'SYS'} = $Sys;
+	$this->{'USER'} = [];
 	
-	undef @{$this->{'USER'}};
-	$path = $Sys->Get('BBSPATH') . '/' . $Sys->Get('BBS') . "/info/access.cgi";
+	my $path = $Sys->Get('BBSPATH') . '/' . $Sys->Get('BBS') . "/info/access.cgi";
 	
-	if (-e $path) {
-		open USER, "< $path";
-		@datas = <USER>;
-		close USER;
+	if (open(my $fh, '<', $path)) {
+		flock($fh, 2);
+		my @datas = <$fh>;
+		close($fh);
+		map { s/[\r\n]+\z// } @datas;
 		
-		($dummy, @datas) = @datas;
-		chomp $dummy;
-		@head = split(/<>/, $dummy);
+		my @head = split(/<>/, shift(@datas), -1);
 		$this->{'TYPE'} = $head[0];
 		$this->{'METHOD'} = $head[1];
 		
-		foreach (@datas) {
-			chomp $_;
-			push @{$this->{'USER'}}, $_;
-		}
+		push @{$this->{'USER'}}, @datas;
 		return 0;
 	}
 	return 1;
@@ -87,7 +69,7 @@ sub Load
 #
 #	[Uf[^ - Save
 #	-------------------------------------------
-#	@F$M : MELKOR
+#	@F$Sys : MELKOR
 #	߂lF폑:0,G[:-1
 #
 #------------------------------------------------------------------------------------------------------------
@@ -95,25 +77,25 @@ sub Save
 {
 	my $this = shift;
 	my ($Sys) = @_;
-	my ($path);
 	
-	$path	= $Sys->Get('BBSPATH') . '/' . $Sys->Get('BBS') . "/info/access.cgi";
+	my $path = $Sys->Get('BBSPATH') . '/' . $Sys->Get('BBS') . "/info/access.cgi";
 	
-#	eval
-	{
-		open USER, "> $path";
-		flock USER, 2;
-		binmode USER;
-		print USER $this->{'TYPE'} . '<>' . $this->{'METHOD'} . "\n";
+	chmod($Sys->Get('PM-ADM'), $path);
+	if (open(my $fh, (-f $path ? '+<' : '>'), $path)) {
+		flock($fh, 2);
+		seek($fh, 0, 0);
+		binmode($fh);
+		
+		print $fh "$this->{'TYPE'}<>$this->{'METHOD'}\n";
 		foreach (@{$this->{'USER'}}) {
-			print USER "$_\n";
+			print $fh "$_\n";
 		}
-		close USER;
-		chmod $Sys->Get('PM-ADM'), $path;
-	};
-	if ($@ ne '') {
-		return $@;
+		
+		truncate($fh, tell($fh));
+		close($fh);
 	}
+	chmod($Sys->Get('PM-ADM'), $path);
+	
 	return 0;
 }
 
@@ -146,9 +128,8 @@ sub Get
 {
 	my $this = shift;
 	my ($key, $default) = @_;
-	my ($val);
 	
-	$val = $this->{$key};
+	my $val = $this->{$key};
 	
 	return (defined $val ? $val : (defined $default ? $default : undef));
 }
@@ -165,7 +146,7 @@ sub Clear
 {
 	my $this = shift;
 	
-	undef @{$this->{'USER'}};
+	$this->{'USER'} = [];
 }
 
 #------------------------------------------------------------------------------------------------------------
@@ -190,72 +171,81 @@ sub Set
 #	[U - Check
 #	-------------------------------------------
 #	@F$host : zXg
+#			$addr : IPAhX
+#			$koyuu : [ŗLʎq
 #	߂lFo^[U:1,o^[U:0
 #
 #------------------------------------------------------------------------------------------------------------
 sub Check
 {
 	my $this = shift;
-	my ($host, $addr) = @_;
-	my ($flag, $sys, $addrb, $adex);
+	my ($host, $addr, $koyuu) = @_;
 	
-	$sys = $this->{'SYS'};
-	$addrb = unpack('B32', pack('C*', split(/\./, $addr)));
-	$flag = 0;
-	$adex = '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}';
+	my $Sys = $this->{'SYS'};
+	my $addrb = unpack('B32', pack('C*', split(/\./, $addr)));
+	my $flag = 0;
+	my $adex = '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}';
 	
 	foreach my $line (@{$this->{'USER'}}) {
-		next if ($line eq '' || $line =~ /^[#;]/);
+		next if ($line =~ /^[#;]|^$/);
 		
+		# IPAhX/CIDR
 		if ($line =~ m|^($adex)(?:/([0-9]+))?$|) {
-			my ($leng, $a);
-			$leng = 32;
-			$leng = $2 if ($2);
-			$a = unpack("B$leng", pack('C*', split(/\./, $1)));
+			my $leng = $2 || 32;
+			my $a = unpack("B$leng", pack('C*', split(/\./, $1)));
 			if (substr($addrb, 0, $leng) eq $a) {
 				$flag = 1;
-				$sys->Set('HITS', $line);
+				$Sys->Set('HITS', $line);
 				last;
 			}
 		}
+		# IPAhX͈͎w
 		elsif ($line =~ m|^($adex)-($adex)$|) {
-			my ($a, $b);
-			$a = unpack('B32', pack('C*', split(/\./, $1)));
-			$b = unpack('B32', pack('C*', split(/\./, $2)));
-			if ($a gt $b) {
-				$_ = $a;
-				$a = $b;
-				$b = $_;
-			}
+			my $a = unpack('B32', pack('C*', split(/\./, $1)));
+			my $b = unpack('B32', pack('C*', split(/\./, $2)));
+			($b, $a) = ($a, $b) if ($a gt $b);
 			if ($addrb ge $a && $addrb le $b) {
 				$flag = 1;
-				$sys->Set('HITS', $line);
+				$Sys->Set('HITS', $line);
 				last;
 			}
 		}
+		# [ŗLʎq
+		elsif (defined $koyuu && $koyuu =~ /^\Q$line\E$/) {
+			$flag = 1;
+			$Sys->Set('HITS', $line);
+			last;
+		}
+		# zXg(K\)
 		elsif ($host =~ /$line/) {
 			$flag = 1;
-			$sys->Set('HITS', $line);
+			$Sys->Set('HITS', $line);
 			last;
 		}
 	}
 	
-	if ($flag && $this->{'TYPE'} eq 'disable') {		# K[U
-		if ($this->{'METHOD'} eq 'disable') {			# Fݕs
+	# K[U
+	if ($flag && $this->{'TYPE'} eq 'disable') {
+		if ($this->{'METHOD'} eq 'disable') {
+			# Fݕs
 			return 4;
 		}
-		elsif ($this->{'METHOD'} eq 'host') {			# FzXg\
+		elsif ($this->{'METHOD'} eq 'host') {
+			# FzXg\
 			return 2;
 		}
 		else {
 			return 4;
 		}
 	}
-	elsif (! $flag && $this->{'TYPE'} eq 'enable') {	# 胆[UȊO
-		if ($this->{'METHOD'} eq 'disable') {			# Fݕs
+	# 胆[UȊO
+	elsif (! $flag && $this->{'TYPE'} eq 'enable') {
+		if ($this->{'METHOD'} eq 'disable') {
+			# Fݕs
 			return 4;
 		}
-		elsif ($this->{'METHOD'} eq 'host') {			# FzXg\
+		elsif ($this->{'METHOD'} eq 'host') {
+			# FzXg\
 			return 2;
 		}
 		else {
