@@ -192,6 +192,9 @@ sub ConvertURL
 					elsif ($cushion =~ m{^(?:jump\.x0\.to|nun\.nu)/$}) {
 						$$text =~ s|$reg2|<a href="http://$cushion$1://$2" target="_blank">$1://$2</a>|;
 					}
+					else {
+						$$text =~ s|$reg2|<a href="$1://$2" target="_blank">$1://$2</a>|;
+					}
 				}
 			}
 		}
@@ -641,6 +644,25 @@ sub MakeID
 	
 	return $ret;
 }
+sub MakeIDnew
+{
+	my $this = shift;
+	my ($Sys, $column) = @_;
+	
+	require Digest::SHA::PurePerl;
+	my $ctx = Digest::SHA::PurePerl->new;
+	$ctx->add('0ch+ ID Generation');
+	$ctx->add(':', $Sys->Get('SERVER'));
+	$ctx->add(':', $Sys->Get('BBS'));
+	$ctx->add(':', $Sys->Get('KOYUU'));
+	$ctx->add(':', join('-', (localtime)[3,4,5]));
+	#$ctx->add(':', $Sys->Get('KEY'));
+	
+	my $id = $ctx->b64digest;
+	$id = substr($id, 0, $column);
+	
+	return $id;
+}
 
 #------------------------------------------------------------------------------------------------------------
 #
@@ -958,7 +980,7 @@ sub GetIDPart
 		if (!$noslip && $Set->Equal('BBS_SLIP', 'checked')) {
 			$str .= " $type";
 		}
-		return " HOST:$str";
+		return "HOST:$str";
 	}
 	
 	# IP\ Ver.Siberia
@@ -974,7 +996,7 @@ sub GetIDPart
 		if (!$noslip && $Set->Equal('BBS_SLIP', 'checked')) {
 			$str .= " $type";
 		}
-		return " M:$str";
+		return "M:$str";
 	}
 	
 	# IP\ Ver.Sakhalin
@@ -992,7 +1014,7 @@ sub GetIDPart
 		if (!$noslip && $Set->Equal('BBS_SLIP', 'checked')) {
 			$str .= " $type";
 		}
-		return " M:$str";
+		return "M:$str";
 	}
 	
 	# eLbvpID
@@ -1001,7 +1023,7 @@ sub GetIDPart
 		if (!$noslip && $Set->Equal('BBS_SLIP', 'checked')) {
 			$str .= " $type";
 		}
-		return " ID:$str";
+		return "ID:$str";
 	}
 	
 	# ID\
@@ -1019,7 +1041,7 @@ sub GetIDPart
 		if (!$noslip && $Set->Equal('BBS_SLIP', 'checked')) {
 			$str .= "$type";
 		}
-		return " ID:$str";
+		return "ID:$str";
 	}
 }
 
